require 'minitest/autorun'
require 'postal_code'

class PostalCodeTest < Minitest::Test

  ProductionDB = PostalCode::DB.dup
  TestDB = File.dirname(__FILE__) + '/data/US.tsv'
  PostalCode::DB.replace(TestDB)

  def test_database_format
    [ProductionDB, TestDB].each do |db|
      [ %x[head -n1 #{db}],
        %x[tail -n1 #{db}]
      ].each do |tuple|
        assert_match(/^\d{5}\t.+\t[A-Z]{2}$/, tuple)
      end
    end
  end

  def test_postal_code_with_leading_zeros
    pcode = '00001'
    assert_equal 'City1', PostalCode.city(pcode)
    assert_equal 'AA',    PostalCode.state(pcode)
  end

  def test_postal_code_with_plus_4_extension
    pcode = '00001-1234'
    assert_equal 'City1', PostalCode.city(pcode)
    assert_equal 'AA',    PostalCode.state(pcode)
  end

  def test_postal_code_found_in_single_city
    pcode = '10000'
    assert_equal 'City2', PostalCode.city(pcode)
    assert_equal 'BA',    PostalCode.state(pcode)
  end

  def test_postal_code_found_in_multiple_cities
    pcode = '20000'
    assert_equal 'City4', PostalCode.city(pcode)
    assert_equal 'CA',    PostalCode.state(pcode)
  end

  def test_postal_code_not_found_in_any_city
    pcode = '99999'
    assert_nil PostalCode.city(pcode)
    assert_nil PostalCode.state(pcode)
  end

  def test_reactive_cache
    PostalCode.clear_cache
    assert_equal 0, PostalCode.instance_variable_get(:@cache).size
    PostalCode.city('00001') # hit
    assert_equal 1, PostalCode.instance_variable_get(:@cache).size

    PostalCode.clear_cache
    assert_equal 0, PostalCode.instance_variable_get(:@cache).size
    PostalCode.city('99999') # miss
    assert_equal 0, PostalCode.instance_variable_get(:@cache).size
  end

  def test_proactive_cache
    tuple_cnt      = open(PostalCode::DB) {|db| db.readlines.size}
    duplicate_cnt  = 1
    uniq_tuple_cnt = tuple_cnt - duplicate_cnt

    assert_equal 4, tuple_cnt
    PostalCode.clear_cache
    assert_equal 0, PostalCode.instance_variable_get(:@cache).size
    PostalCode.load_cache
    assert_equal uniq_tuple_cnt, PostalCode.instance_variable_get(:@cache).size
  end

  def test_postal_code_validations
    assert PostalCode.valid_format?('01234-1234')
    refute PostalCode.valid_format?('01234-123')
    refute PostalCode.valid_format?('01234-12')
    refute PostalCode.valid_format?('01234-1')
    refute PostalCode.valid_format?('01234-')
    assert PostalCode.valid_format?('01234')
    refute PostalCode.valid_format?('0123')
    refute PostalCode.valid_format?('012')
    refute PostalCode.valid_format?('01')
    refute PostalCode.valid_format?('0')
  end

  def test_postal_code_validation_exceptions
    assert_raises(TypeError) { PostalCode.valid_format? 99999 }
    assert_raises(TypeError) { PostalCode.valid_format? 00001 }
    assert_raises(TypeError) { PostalCode.valid_format? 1 }
  end

end
